;PVdBDownPage.ahk 
;AutoHotKey script for Personal Video Database download page to file UTF-8 NO BOM:
;   First argument A_Args[1]= URL
;   Second argument A_Args[2]= TargetFile
;   Third argument A_Args[3]= If present shows the script variables
;   If exists 'PVdBDownPage.log' write variables in this file
;CHANGE LOG:
;   V.2018.05.06-VVV_Easy: Add log funtion in 'PVdBDownPage.log'.
;   V.2018.05.05-VVV_Easy: First version.
;------------------------------------------------------------
;Script behavior :
;   1) Hides the windows and runs in background
;   2) Encode the URL in escaped %Hexadecimal codes (for instance Space=%20)
;   3) Deletes the target file
;   4) Download the page in a UTF-8 NO BOM text file
;------------------------------------------------------------
;Compilation orders
    #NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
    #Warn  ; Enable warnings to assist with detecting common errors.
    #SingleInstance On  ;The word OFF allows multiple instances of the script to run concurrently
    ;#NoTrayIcon ;Disables the showing of a tray icon.
;Script behavior and Global variables
    Global ScriptName 
    SplitPath, A_ScriptName,,,, ScriptName ;Get ScriptName for windows title and log file.
    ;SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
    Global s_LogFile:= A_ScriptDir . "\PVdBDownPage.log"
    ;If not needed arguments show help
    If A_Args.Length()<2 { 
        MsgBox, 16, % ScriptName, % ScriptName ":`nAutoHotKey script for Personal Video Database download page to file UTF-8 NO BOM.`n`nUsage:`nFirst argument = URL`nSecond argument = TargetFile`nThird argument = If present shows the script variables`n`nIf exists 'PVdBDownPage.log', write variables in this file`n"
        ExitApp
    }
    WinHide, %A_ScriptFullPath%
    WinHide, %A_ScriptName%
    Global EscapedURL:=url_encode(A_Args[1])
    StringReplace, EscapedURL, EscapedURL, `", / , All ;If the URL finish in / provoques a escape quote. See https://www.autohotkey.com/docs/Scripts.htm#cmd
    ;Show variables with debug third parameter
    If A_Args.Length()>2
        MsgBox, 64, % ScriptName, % ScriptName " variables:`n`nNumber of parameters = " A_Args.Length() " `nURL = |" A_Args[1] "| `nFile = |" A_Args[2] "| `nEscapedURL = |" EscapedURL "| `n`t"
    ;If exists, write variables in the log file
    If FileExist(s_LogFile)
        try {  ;Asegurate file access
            FileAppend % A_Now "-" ScriptName " variables:`n`tNumber arguments = " A_Args.Length() "`n`t       URL=|" A_Args[1] "|`n`tEscapedURL=|" EscapedURL "|`n`tFile=|" A_Args[2] "| `n", %s_LogFile%
		}
        catch e {
            MsgBox 16, % A_ThisFunc . "-(" . A_LineNumber . ")", % "Error catch " e.What ", which was called at line " e.Line ", in:`n|" s_LogFile "|"
            ExitApp            
        }
    If FileExist(A_Args[2])
        FileDelete, A_Args[2] 
    Download(EscapedURL,A_Args[2],True,{"User-Agent": "Mozilla/5.0 (Windows NT 6.3; WOW64; rv:37.0) Gecko/20100101 Firefox/37.0", "Cache-Control": "max-age=0"})
    ExitApp
;SUBRUTINES--------------------------------------------------
;AutoHotKey Snipets------------------------------------------------------------------------
url_encode(str){ ; encode the special characters of a UTF-8 string in a string. Some chars (-._~:/?#@!$&()[]*+=,;%) are omited because are reserved for url in the RFC 1738 specification.
    ; https://autohotkey.com/board/topic/17367-url-encoding-and-decoding-of-special-characters/#entry501592
    ;Usage-> URL:=fn_encode(string); string:=fn_decode(URL)
    ;Documentation: Characters allowed in a URL https://stackoverflow.com/questions/1856785/characters-allowed-in-a-url
    ;   IPv4address = dec-octet "." dec-octet "." dec-octet "." dec-octet
    ;   dec-octet   = DIGIT                 ; 0-9
    ;          / %x31-39 DIGIT         ; 10-99
    ;          / "1" 2DIGIT            ; 100-199
    ;          / "2" %x30-34 DIGIT     ; 200-249
    ;          / "25" %x30-35          ; 250-255
    ;   reg-name    = *( unreserved / pct-encoded / sub-delims )
    ;       unreserved  = ALPHA / DIGIT / "-" / "." / "_" / "~"     <---This seems like a practical shortcut, most closely resembling original answer
    ;       reserved    = gen-delims / sub-delims
    ;       gen-delims  = ":" / "/" / "?" / "#" / "[" / "]" / "@"
    ;       sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
    ;                     / "*" / "+" / "," / ";" / "="
    ;       pct-encoded = "%" HEXDIG HEXDIG
   f = %A_FormatInteger%
   SetFormat, Integer, Hex   ; set integer format to hex
   
   If RegExMatch(str, "^\w+:/{0,2}", pr)   
      StringTrimLeft, str, str, StrLen(pr)
   
   ;StringReplace, str, str, `%, `%25, All    ; replace all % with %25 -> Bad for some url with decoded chars
   
   Loop
      If RegExMatch(str, "i)[^\w\-._~:/?#@!$&()\[\]*+=,;%]", char)    ; exclude alphnumeric -._~:/?#@!$&()[]*+=,;%
         StringReplace, str, str, %char%, % "%" . fn_zerofill(SubStr(Asc(char),3),2) , All
      Else Break
   
   SetFormat, Integer, %f%   ; restore integer format
   Return, pr . str
}
url_decode(str){
    Loop
        If RegExMatch(str, "i)(?<=%)[\da-f]{1,2}", hex)
            StringReplace, str, str, `%%hex%, % Chr("0x" . hex), All
        Else Break
    Return, str
}
fn_zerofill(num, size){   ; returns num zerofilled to size digits ; example call to zerofill -> n := zerofill(n, 3)
    StringLen, length, num
    c := size - length 
    loop, %c%
        num := "0" num
    return num
}
;==========================================================================================
Download(UrlToFile,SaveFileAs:="",Overwrite:=True,headers:="",method:="GET",postData:="") { ; https://stackoverflow.com/questions/29460945/alternative-to-urldownloadtofile 
    ;Examples:
    ;   1)    Download("http://ahkscript.org/download/1.1/AutoHotkey111402_Install.exe")
    ;   2)    customHeaders := {"User-Agent": "Mozilla/5.0 (Windows NT 6.3; WOW64; rv:37.0) Gecko/20100101 Firefox/37.0", "Cache-Control": "max-age=0", "Cookie": "downloadtoken=b82416fdb23e421fb5a"}
    ;         Download("http://download.piriform.com/ccsetup410.exe","",True,customHeaders)
    ;   3)    Download("http://foo.bar/example.exe","example.exe",True,{"Cookie":"sessionid=abc123"},"POST","username=foo_bar&password=qwerty")
    ;
    WinHttpObj := ComObjCreate("WinHttp.WinHttpRequest.5.1")
    try { ;Script fails in not good url
        WinHttpObj.Open(method, UrlToFile)
    }
    catch e2 {
            ;MsgBox 16, % A_ThisFunc . "-(" . A_LineNumber . ")", % "Error catch " e2.What ", which was called at line " e2.Line ", in:`n|" UrlToFile "|"
            ;ExitApp
            If FileExist(s_LogFile)
                try {  ;Asegurate file access
                    FileAppend % A_Now "-" ScriptName "-Error catch " e2.What ", which was called at line " e2.Line ", in:|" UrlToFile "|`n", %s_LogFile%
                }
                catch e3 {
                    MsgBox 16, % A_ThisFunc . "-(" . A_LineNumber . ")", % "Error catch " e3.What ", which was called at line " e3.Line ", in:`n|" s_LogFile "|"
                    ExitApp            
                }            
            Return            
    }
    For header, value in headers 
        WinHttpObj.SetRequestHeader(header, value)
    WinHttpObj.Send(postData)

    ADODBObj := ComObjCreate("ADODB.Stream")
    ADODBObj.Type := 1
    ADODBObj.Open()
    ADODBObj.Write(WinHttpObj.ResponseBody)
    If !SaveFileAs {
        urlSplitArray := StrSplit(UrlToFile, "/")
        SaveFileAs := urlSplitArray[urlSplitArray.MaxIndex()]
    }        
    ADODBObj.SaveToFile(SaveFileAs, Overwrite ? 2:1)
    ADODBObj.Close()
}
;==========================================================================================

